module MD5
    (
      State (..),
      Block (..),
      hex_hash,
      text_to_bytestring,
      md5_bytestring,
      md5_text,
      md5_file
    )
  where

import Data.List
import qualified Data.ByteString.Lazy as B

import MD5.Types
import MD5.Constants
import MD5.Compress
import MD5.Pad

text_to_bytestring :: String -> B.ByteString
text_to_bytestring = B.pack . map (fromIntegral . fromEnum)

md5_bytestring :: B.ByteString -> State
md5_bytestring = foldl' compress_block initial_state . pad

md5_text :: String -> String
md5_text = hex_hash . md5_bytestring . text_to_bytestring

md5_file :: FilePath -> IO String
md5_file f = do
  bs <- B.readFile f
  return $ hex_hash $ md5_bytestring bs
