module MD5.Constants
    (
      block_size_bits,
      block_size_bytes,
      block_size_words,
      initial_state,
      round_table
    )
  where

import Data.Bits
import Data.Word
import Data.List

import MD5.Types

block_size_bits  = 512                       :: Word64
block_size_bytes = block_size_bits  `div`  8 :: Word64
block_size_words = block_size_words `div` 32 :: Word64

initial_state = State 0x67452301 0xEFCDAB89 0x98BADCFE 0x10325476

shift_table :: [Int]
shift_table =
  (concat . replicate 4) [07, 12, 17, 22] ++
  (concat . replicate 4) [05, 09, 14, 20] ++
  (concat . replicate 4) [04, 11, 16, 23] ++
  (concat . replicate 4) [06, 10, 15, 21]

sin_table :: [Word32]
sin_table = map (\x -> floor(abs(sin(x + 1)) * (2^32))) [0..63]

f_table :: [State -> Word32]
f_table =
  replicate 16 (\(State a b c d) -> (b .&. c) .|. ((complement b) .&. d)) ++
  replicate 16 (\(State a b c d) -> (d .&. b) .|. ((complement d) .&. c)) ++
  replicate 16 (\(State a b c d) -> b `xor` c `xor` d)                    ++
  replicate 16 (\(State a b c d) -> c `xor` (b .|. (complement d)))

index_table :: [Word8]
index_table =
  map (\i -> i)                  [ 0..15] ++
  map (\i -> (5*i + 1) `mod` 16) [16..31] ++
  map (\i -> (3*i + 5) `mod` 16) [32..47] ++
  map (\i -> (7*i + 0) `mod` 16) [48..63]

round_table = zip4 shift_table sin_table f_table index_table
